import pandas as pd
import xlrd
import sys
from tkinter import filedialog
from tkinter import *
import math

COLUMN_TYPE=0
COLUMN_NOM_OUT=2
COLUMN_NUM_OUT=3
COLUMN_DIAMETRE=4
COLUMN_LONGUEUR_COUPE=5
COLUMN_LONGUEUR_TOTALE=6
COLUMN_NBRE_DENTS=7
COLUMN_DIA_QUEUE=10

class bcolors:
    OK = '\033[92m' #GREEN
    WARNING = '\033[93m' #YELLOW
    FAIL = '\033[91m' #RED
    RESET = '\033[0m' #RESET COLOR

# ---------------- recherche_default_path ---------------------

def recherche_default_path():

    #  recherche le path par defaut si il existe ==> fichier dans le repertoire du python nommé DEFAULT_PATH
    #  creation du fichier DEFAULT_PATH si il n'existe pas pointant vers cwd

    import os.path
    dir_path = os.path.dirname(os.path.realpath(__file__))
    fichier_path = os.path.join(dir_path, "DEFAULT_PATH")

    fichier_exist = os.path.exists(fichier_path)
    if fichier_exist:
        with open(fichier_path, 'r',encoding='utf-8') as input_file:
            for path in input_file:
                if  os.path.exists(path):
                    dir_path = path
                else:
                    dir_path = os.getcwd()
    else:
        with open(fichier_path, "w", encoding='utf-8') as sortie:
            dir_path = os.getcwd()
            sortie.write(f"{dir_path}")

    return dir_path


# ---------------- verif_type ---------------------
#          verifie si le type d'outil est valide
#

def verif_type(ligne):

    Ligne_low = ligne.lower()

    type_found = -1
    if "fraise cylindrique" in Ligne_low:
        type_found = 0
    if "fraise sphérique" in Ligne_low:
        type_found = 1
    if "fraise conique" in Ligne_low:
        type_found = 2
    if "fraise de forme" in Ligne_low:
        type_found = 3
    if "foret bois" in Ligne_low:
        type_found = 4
    if "foret" in Ligne_low:
        type_found = 5
    if "foret à centrer" in Ligne_low:
        type_found = 6

    return type_found
# ---------------- check_data ---------------------
#           verifie si ue ligne outil est bien formée
#           retourne -1 si ce n'est pas bon sinon retourne:
#                                               0 pour un type fraise cylindrique
#                                               1 pour un type fraise fraise sphérique
#                                               2 pour un type fraise conique
#                                               3 pour un type fraise fraise de forme
#                                               4 pour un type foret bois
#                                               5 pour un type foret
#                                               6 pour un type foret à centrer

def check_data(ligne,numero_ligne):

    global COLUMN_TYPE
    global COLUMN_NUM_OUT
    global COLUMN_DIAMETRE
    global COLUMN_LONGUEUR_COUPE
    global COLUMN_LONGUEUR_TOTALE
    global COLUMN_NBRE_DENTS
    global COLUMN_DIA_QUEUE

    type_verifie = verif_type(ligne[COLUMN_TYPE])
#  verification que le type est bon
    if type_verifie < 0:
        print (bcolors.FAIL + " le type d'outil de la ligne " + str(numero_ligne) + " est mauvais. Il ne peut prendre que les valeurs suivantes : Fraise Cylindrique, Fraise Sphérique, Fraise Conique, Fraise de forme, Foret bois, Foret, Foret à centrer" )

#  verification que le numéro commence par puis un numerique
    if not ligne[COLUMN_NUM_OUT].find("T"):
        index= ligne[COLUMN_NUM_OUT].replace("T", "")
        if not index.isnumeric:
            print(bcolors.FAIL + "Le numéro d'outil de la ligne " + str(numero_ligne) + " devrait etre de la forme T(numero outil) avec numero d'outil = un entier")
            type_verifie = -1
    else:
        print("bcolors.FAIL + Le numéro d'outil de la ligne " + str(numero_ligne) + " devrait etre de la forme T(numero outil)")
        type_verifie = -1

    #  verification que diametre est un numerique
    if math.isnan(ligne[COLUMN_DIAMETRE]):
        print(bcolors.FAIL + "Le diametre d'outil de la ligne " + str(numero_ligne) + " n'est pas un numerique")
        type_verifie = -1

    #  verification que la longueur de coupe est un numerique
    if math.isnan(ligne[COLUMN_LONGUEUR_COUPE]):
        print(bcolors.FAIL + "La longueur de coupe de l'outil de la ligne " + str(numero_ligne) + " n'est pas un numerique")
        type_verifie = -1

    #  verification que la longueur totale est un numerique
    if math.isnan(ligne[COLUMN_LONGUEUR_TOTALE]):
        print(bcolors.FAIL + "La longueur totale de l'outil de la ligne " + str(numero_ligne) + " n'est pas un numerique")
        type_verifie = -1

    #  verification que le nombre de dents est un numerique

    if math.isnan(ligne[COLUMN_NBRE_DENTS]):
        print(bcolors.FAIL + "Le nombre de dents de l'outil de la ligne " + str(numero_ligne) + " est vide")
        type_verifie = -1

    #  verification que le diametre de la queue est un numerique

    if math.isnan(ligne[COLUMN_DIA_QUEUE]):
        print(bcolors.FAIL + "Le diametre de la queue de l'outil de la ligne " + str(numero_ligne) + " est vide")
        type_verifie = -1

    return type_verifie
# ---------------- set_default_path ---------------------

def set_default_path(fichier_choisi):

    #   set le default path en fonction de la derniere recherche de fichier

    import os.path
    path_fichier = os.path.dirname(fichier_choisi)

    path_executable = os.path.dirname(os.path.realpath(__file__))
    fichier_path = os.path.join(path_executable, "DEFAULT_PATH")

    with open(fichier_path, "w", encoding='utf-8') as sortie:
        sortie.write(f"{path_fichier}")

# ------------------------------- Construit_camban_file(df)   -----------------------
#       construction du fichier outil pour cambam
#       input df = resultat de la lecture du fichier excel
#             path = chemin d'acces par defaut du fichier camban

def Construit_camban_file(df,path):

    global COLUMN_TYPE
    global COLUMN_NOM_OUT
    global COLUMN_NUM_OUT
    global COLUMN_DIAMETRE
    global COLUMN_LONGUEUR_COUPE
    global COLUMN_LONGUEUR_TOTALE
    global COLUMN_NBRE_DENTS
    global COLUMN_DIA_QUEUE

    nbr_ligne = df.index.stop
    nbr_colonne = len(df.columns)

    fichier_sortie = filedialog.asksaveasfilename(initialdir=path, title="Selectionnez votre fichier de sortie outils cambam (extension .xml)", filetypes=[("Fichier xml" , ".xml")])

    if fichier_sortie.find(".xml"):

        LINE_OUT =  "<?xml version=\"1.0\" encoding=\"utf-8\"?>"  + chr(10)
        LINE_OUT =  LINE_OUT + "<ToolLibrary Version=\"0.9.8.0\" NumberFormat=\"0.00\">"  + chr(10)

        numero_ligne = 1
        for col in range(nbr_ligne):

            numero_ligne = numero_ligne + 1
            type_verifie = check_data(df.values[col],numero_ligne)

            if type_verifie >=0 :

                type_valide = ["EndMill","BallNose","VCutter","EndMill","Drill","Drill","Drill"]
                type_cambam=type_valide [type_verifie]
                index= df.values[col][COLUMN_NUM_OUT].replace("T", "")
                LINE_OUT = LINE_OUT + "<ToolDefinition Name=\"" + df.values[col][COLUMN_NOM_OUT] + "\">" + chr(10)
                LINE_OUT = LINE_OUT + "<Index>" + str(index) + "</Index>" + chr(10)
                LINE_OUT = LINE_OUT + "<Diameter>" + str(df.values[col][COLUMN_DIAMETRE]) + "</Diameter>" + chr(10)
                LINE_OUT = LINE_OUT + "<ToolProfile>" + type_cambam + "</ToolProfile>" + chr(10)
                LINE_OUT = LINE_OUT + "<Flutes>" + str(int(df.values[col][COLUMN_NBRE_DENTS])) + "</Flutes>" + chr(10)
                LINE_OUT = LINE_OUT + "<FluteLength>" + str(df.values[col][COLUMN_LONGUEUR_COUPE]) + "</FluteLength>" + chr(10)
                LINE_OUT = LINE_OUT + "<Length>" + str(df.values[col][COLUMN_LONGUEUR_TOTALE]) + "</Length>" + chr(10)
                LINE_OUT = LINE_OUT + "<PartCode>" + df.values[col][COLUMN_NUM_OUT] + "</PartCode>" + chr(10)
                LINE_OUT = LINE_OUT + "<ShankDiameter>" + str(df.values[col][COLUMN_DIA_QUEUE]) + "</ShankDiameter>" + chr(10)
                LINE_OUT = LINE_OUT + "<HelixAngle>0</HelixAngle>" + chr(10)
                LINE_OUT = LINE_OUT + "<VeeAngle>0</VeeAngle>" + chr(10)
                LINE_OUT = LINE_OUT + "<MaxRampAngle>0</MaxRampAngle>" + chr(10)
                LINE_OUT = LINE_OUT + "<ToothLoad>0</ToothLoad>" + chr(10)
                LINE_OUT = LINE_OUT + "<AxialDepthOfCut>0</AxialDepthOfCut>" + chr(10)
                LINE_OUT = LINE_OUT + "<RadialDepthOfCut>0</RadialDepthOfCut>" + chr(10)
                LINE_OUT = LINE_OUT + "</ToolDefinition>" + chr(10)

        LINE_OUT =  LINE_OUT + "</ToolLibrary>"

        with open(fichier_sortie, "w", encoding='utf-8') as sortie:
            sortie.write(f"{LINE_OUT}")


        print ("")
        print (bcolors.OK + "Fichier cambam construit : " + fichier_sortie)
        print ("")

    else:
        print(bcolors.FAIL +"L'extension d'un fichier outil cambam doit etre .xml")

# ------------------------------- Construit_camotics_file(df,path)   -----------------------
#       construction du fichier outil pour camotics
#       input df = resultat de la lecture du fichier excel
#             path = chemin d'acces par defaut du fichier camotics

def Construit_camotics_file(df,path):

    global COLUMN_NOM_OUT
    global COLUMN_NUM_OUT
    global COLUMN_DIAMETRE
    global COLUMN_LONGUEUR_TOTALE

    nbr_ligne = df.index.stop
    nbr_colonne = len(df.columns)

    fichier_sortie = filedialog.asksaveasfilename(initialdir=path, title="Selectionnez votre fichier de sortie outils camotics (extension .json)", filetypes= [("Fichier json" , ".json" )])

    if fichier_sortie.find(".json"):

        LINE_OUT =  "{" + chr(10)

        numero_ligne = 1
        for col in range(nbr_ligne):

            numero_ligne = numero_ligne + 1
            type_verifie = check_data(df.values[col],numero_ligne)

            if type_verifie >=0 :

                type_valide = ["cylindrical","ballnose","conical","cylindrical","cylindrical","cylindrical","cylindrical"]
                type_camotics=type_valide [type_verifie]
                index= df.values[col][COLUMN_NUM_OUT].replace("T", "")
                LINE_OUT = LINE_OUT + "  \"" + str(index)  + "\": {" + chr(10)
                LINE_OUT = LINE_OUT + "    \"units\": \"metric\"," + chr(10)
                LINE_OUT = LINE_OUT + "    \"shape\": \"" + type_camotics + "\"," + chr(10)
                LINE_OUT = LINE_OUT + "    \"length\": " + str(df.values[col][COLUMN_LONGUEUR_TOTALE]) + "," + chr(10)
                LINE_OUT = LINE_OUT + "    \"diameter\": " + str(df.values[col][COLUMN_DIAMETRE]) + "," + chr(10)
                LINE_OUT = LINE_OUT + "    \"description\": \"" + df.values[col][COLUMN_NOM_OUT] + "\"" + chr(10)
                LINE_OUT = LINE_OUT + "  }," + chr(10)


        LINE_OUT =  LINE_OUT + "}"

        with open(fichier_sortie, "w", encoding='utf-8') as sortie:
            sortie.write(f"{LINE_OUT}")

        print ("")
        print (bcolors.OK +"Fichier camotics construit : " + fichier_sortie)
        print ("")
    else:
        print(bcolors.FAIL + "L'extension d'un fichier outil camotics doit etre .json")
# ------------------------------- Construit_linuxcnc_file(df,path)   -----------------------
#       construction du fichier outil pour linuxcnc
#       input df = resultat de la lecture du fichier excel
#             path = chemin d'acces par defaut du fichier linuxcnc

def Construit_linuxcnc_file(df,path):

    global COLUMN_NOM_OUT
    global COLUMN_NUM_OUT
    global COLUMN_DIAMETRE
    global COLUMN_LONGUEUR_COUPE
    global COLUMN_LONGUEUR_TOTALE

    nbr_ligne = df.index.stop
    nbr_colonne = len(df.columns)

    fichier_sortie = filedialog.asksaveasfilename(initialdir=path, title="Selectionnez votre fichier de sortie outils linuxcnc (extension .tbl)", filetypes= [("Fichier tbl" , ".tbl" )])

    if fichier_sortie.find(".tbl"):

        numero_ligne = 1
        LINE_OUT = ""
        for col in range(nbr_ligne):

            numero_ligne = numero_ligne + 1
            type_verifie = check_data(df.values[col],numero_ligne)

            if type_verifie >=0 :
                LINE_OUT = LINE_OUT + df.values[col][COLUMN_NUM_OUT] + " P" + str(int(round(df.values[col][COLUMN_LONGUEUR_TOTALE]))) + " Z" + str(int(round(df.values[col][COLUMN_LONGUEUR_COUPE])))+ " D" + str(df.values[col][COLUMN_DIAMETRE]) + " ;" + df.values[col][COLUMN_NOM_OUT] + chr(10)

        with open(fichier_sortie, "w", encoding='utf-8') as sortie:
            sortie.write(f"{LINE_OUT}")

        print ("")
        print (bcolors.OK + "Fichier linuxcnc construit : " + fichier_sortie)
        print ("")
    else:
        print(bcolors.FAIL + "L'extension d'un fichier outil linuxcnc doit etre .tbl")
# ------------------------main -----------------------------
#
# selection du fichier excel
#
path = recherche_default_path()
excel = Tk()
excel.filename = filedialog.askopenfilename (initialdir=path, title="Selectionnez votre fichier outil excel", filetypes=(("excel files", "*.xlsx"), ("all files", "*.*")))

if excel.filename.find(".xlsx"):
    df = pd.read_excel(excel.filename,0)

    traite =0

    if len(sys.argv) == 1:
        ALL = True
    elif len(sys.argv) == 2:
        if sys.argv[1] == "all":
            ALL = True
        else:
            ALL = False
            if sys.argv[1] == "cambam":
                traite = 1
            if sys.argv[1] == "camotics":
                traite = 2
            if sys.argv[1] == "linuxcnc":
                traite = 3
    else:
        print(bcolors.FAIL + "Syntaxe py fic_out_from_excel.py all ou cambam ou linuxcnc ou camotics")

    if ALL or traite == 1:
        Construit_camban_file(df,path)

    if ALL or traite == 2:
        Construit_camotics_file(df,path)

    if ALL or traite == 3:
        Construit_linuxcnc_file(df,path)
else:
    print (bcolors.FAIL + "L'extension d'un fichier excel doit etre .xlsx")